local min, max, sqrt = math.min, math.max, math.sqrt

-- @param id - A unique id used to reference node
-- @param x  - The x coordinate Node spawned at (optional)
-- @param y  - The y coordinate Node spawned at (optional)
-- @param anchor - Node locked in place or not (optional)

local Node = {}

function Node.new( id, x, y, color, anchor )
    local self = {}

    local x, y		= x or 0, y or 0	-- current node position
    local ax, ay	= 0, 0				-- acceleration
    local vx, vy	= 0, 0				-- velocity
	local anchor	= anchor or false

    local function clamp( mn, val, mx )	-- prevents graph 'exploding'
        return math.max( mn, min( val, mx ) )
		end

    local function applyForce( force )	-- calculate node acceleration
        ax = clamp( -force.max, ax + force.x, force.max )
        ay = clamp( -force.max, ay + force.y, force.max )
		end

	local function getDist( tx, ty )	-- node sepatation - node(x,y) and target(tx,ty)
		local dx, dy = x-tx, y-ty
		return sqrt( dx * dx + dy * dy ) + 0.1
		end

    function self:attractTo( node )		-- inter-nodal attraction
		local tx, ty	= node:getPosition()
        local distance	= getDist( tx, ty )
        local dx, dy	= (x-tx) / distance, (y-ty) / distance
        local strength	= -1 * force.spring * distance * 0.5
		force.x, force.y = dx * strength, dy * strength
		applyForce( force )
		end

    function self:repelFrom( node )	-- node repulsion
		local tx, ty	= node:getPosition()
        local distance	= getDist( tx, ty )
        local dx, dy	= (x-tx) / distance, (y-ty) / distance
        local strength	= force.change * (( param.mass * node:getMass() ) / ( distance * distance ))
        force.x, force.y = dx * strength, dy * strength
		applyForce( force )
		end

    function self:move( dt )		-- update node's position (vel, acc and damp)
        vx		= (vx + ax * dt * param.speed) * param.damping
        vy		= (vy + ay * dt * param.speed) * param.damping
        x, y	= x + vx, y + vy	-- x, y is the actual location of node
        ax, ay	= 0, 0
		end

    function self:getID()		return id		end	-- node name
    function self:getX()		return x		end
    function self:getY()		return y		end
    function self:getPosition() return x, y		end
	function self:isAnchor()	return anchor	end
	function self:getMass()		return param.mass		end
    function self:setPosition( nx, ny ) x, y = nx, ny	end
    function self:Anchor( nanchor ) anchor = nanchor	end

    return self
	end

return Node
