-- Graphoon by Robert Machmer
-- Demo refractured by Ref
local Camera	= require 'CameraX'
local Graph		= require 'Graphoon.Graph'
local menu		= require 'menu'
local gr		= love.graphics

local graph, grabbed, selected, nodeIDs, camera
local floor,sin,cos,pi = math.floor, math.sin, math.cos,math.pi

color = {
	background	= { 160, 180, 190 },
	node		= { 130, 190, 238 },
	edge		= { 100, 100, 100 },
	text		= {   3,  10,  56 },
	cursor		= { 234,  49, 110 },
	center		= { 255,   0,   0 },
	rotation	= { 185, 155,   0 },
	stats		= {   0, 115,   0 },
	calls		= {   0, 155,   0, 180 },
	controls	= { 125, 125, 185, 155 },
	bounding	= { 255,   0,   0, 185 }
	}

sprite	= {
	image			= love.graphics.newImage( 'file.png' ), size = 18 }
	sprite.ht		= sprite.image:getHeight()
	sprite.scale	= sprite.size/sprite.ht
	sprite.offset	= sprite.ht / 2

force			= { spring = 0.005, change  = 20,   max  = 4 }
param			= { speed  = 300,   damping = 0.95, mass = 3 }
rotate			= { angle = 0 }
fontsize		= sprite.size
show 			= {				-- key codes for enabling/disabling features
					l	= true,	-- labels
					e	= true,	-- edges
					p	= true,	-- points (nodes)
					c	= false,-- calls
					r	= false,-- rotation
					b	= false,-- boundingbox
					z	= false,-- stats
					x	= false,-- graph center
					m	= true, -- controls
					u	= false}-- save graph

local data		= { sets = { 'loveRun', 'GraphoonData', 'EvilDead' }, selected = 1 }
local screen	= { wd = 1200, ht = 700 }

function love.load()
    gr.setBackgroundColor( color.background )
	love.window.setMode( screen.wd, screen.ht )
    gr.setFont( gr.newFont( fontsize ) )
	graph	= Graph.new( data.sets[ data.selected ] )
    nodeIDs = 0				-- id for nodes added later
	keys	= { up = {'up'}, down = {'down'}, left = {'left'}, right = {'right'} }
    camera	= Camera.new()	-- for zoom and graph movement
	camera:setKeys( keys )	-- overrides default control key settings
	menu:create()			-- control's menu
	end

function love.update( dt )
    graph:update( dt )
    if grabbed then grabbed:setPosition( camera:mousepos() ) end
	camera:update()
	if show.r then
		rotate.angle = rotate.angle + dt/10
		camera:rotateTo( rotate.angle )
		end
	menu:update( dt )
	if show.u then graph.save( 'Graphoon' ) show.u = false end
	end

function love.draw()
	if show.m then menu:draw( 1030, 210 ) end	-- show key control options
    camera:draw(
		function() graph:draw()
        if selected then
            local mx, my = camera:mousepos()
            gr.line( selected:getX(), selected:getY(), mx, my )
            gr.setColor( color.cursor )
			gr.circle( 'line', mx, my , 30 )
			end
		if show.b then graph.showBoundingBox() end	-- show bounding box
		end
		)
	if show.z then graph.getStats() end		-- show performance info
	if show.x then showOrientation() end	-- show rotation angle
	end

function love.mousepressed( _, _, button )	-- grab & connect nodes
    local mx, my	= camera:mousepos()
    local node		= graph:getNodeAt( mx, my, 30 )-- 30 = how close to node
	if node and not grabbed then
		if button == 1 and not selected then
			grabbed = node
			elseif button == 1 and selected then
				graph:addEdge( selected, node )
				selected = nil
			elseif button == 2 and not selected then
				selected = node
			end
		else
			selected, grabbed = nil, nil
		end
	end

function love.keypressed( key )
    if key == 'space' and grabbed then
        local newNode = graph:addNode( nodeIDs, camera:mousepos() )
        graph:addEdge( grabbed, newNode )
        nodeIDs = nodeIDs + 1
		elseif key == 'backspace' and grabbed then
			graph:removeNode( grabbed )
		elseif key == 'n' then
			data.selected = data.selected < #data.sets and data.selected + 1 or 1
			graph = Graph.new( data.sets[data.selected] )
		elseif pressed( key, show) then show[key] = not show[key]
		elseif key == 'escape'	then love.event.quit()
		end
	end

function pressed( k, list )	-- determines if index k is present in table
	for s,_ in pairs( list ) do if s == k then return true end end
	return false
	end

function showOrientation()
	gr.setColor( color.rotation )
	local cx, cy	= camera:getLocation()
	local rot		= camera:getRotation()
	local c, s		= cos( rot - pi/4 ), sin( rot - pi/4 )
	local sn		= {10*(c-s), 10*(s+c)}
	local x1,y1,x2,y2 = cx+sn[1], cy+sn[2], cx-sn[1], cy-sn[2]
	local x3,y3,x4,y4 = cx-sn[2], cy+sn[1], cx+sn[2], cy-sn[1]
	gr.circle( 'fill', x4, y4, 5 )
	gr.line( x1,y1,x2,y2 )
	gr.line( x3,y3,x4,y4 )
	end
